from typing import Any

from pydantic import BaseModel, PrivateAttr

from gisaf.models.info_item import Tag, InfoItem
from gisaf.models.tags import Tags

# class ActionResult(BaseModel):
#     message: str


# class ActionResults(BaseModel):
#     name: str
#     message: str
#     actionResults: list[ActionResult]


class DataProvider(BaseModel):
    store: str
    name: str
    values: list[str]


class InfoCategory(BaseModel):
    name: str
    infoItems: list[InfoItem]


class PlotBgShape(BaseModel):
    name: str
    valueTop: float
    valueBottom: float
    color: str


class PlotBaseLine(BaseModel):
    name: str
    value: float
    color: str


class PlotParams(BaseModel):
    baseLines: list[PlotBaseLine] = []
    bgShapes: list[PlotBgShape] = []
    barBase: float | None = None


class Attachment(BaseModel):
    name: str
    path: str


class FeatureInfo(BaseModel):
    id: str
    itemName: str
    geoInfoItems: list[InfoItem] = []
    surveyInfoItems: list[InfoItem] = []
    infoItems: list[InfoItem] = []
    categorizedInfoItems: list[InfoCategory] | None = None
    tags: list[Tag] = []
    graph: str | None = None
    plotParams: PlotParams | None = None
    files: list[Attachment] = []
    images: list[Attachment] = []
    externalRecordUrl: str | None = None


class ModelValue(BaseModel):
    name: str
    title: str
    unit: str
    chartType: str = 'line'
    chartColor: str = 'blue'


class FormField(BaseModel):
    name: str
    type: str
    dflt: str | None = None
    value: str | None = None


class ModelAction(BaseModel):
    name: str
    icon: str
    roles: list[str] | None = None
    formFields: list[FormField]


class TagAction(BaseModel):
    name: str
    _plugin: Any
    action: str
    roles: list[str] | None = None
    link: str | None = None
    save: bool = False


class TagActions(BaseModel):
    domain: str
    key: str
    actions: list[TagAction]


class ActionAction(BaseModel):
    roles: list[str]
    # plugin: Any
    name: str
    params: Any
    icon: str
    formFields: list[FormField]


class Downloader(BaseModel):
    _plugin: Any = PrivateAttr()  # DownloadPlugin
    roles: list[str] = []
    name: str
    icon: str | None

    def __init__(self, _plugin, **data):
        super().__init__(**data)
        # We generate the value for our private attribute
        self._plugin = _plugin


class LegendItem(BaseModel):
    key: str
    value: str


class ModelInfo(BaseModel):
    store: str
    modelName: str
    symbol: str | None = None
    values: list[ModelValue] = []
    actions: list[ModelAction] = []
    formName: str | None = None
    formFields: list[FormField] = []
    tagPlugins: list[str] = []
    tagActions: list[TagActions] = []
    downloaders: list[Downloader] = []
    legend: list[LegendItem] = []


class TagsStore(BaseModel):
    store: str
    tagActions: list[TagActions]


class TagsStores(BaseModel):
    stores: list[TagsStore]


class ActionParam(BaseModel):
    name: str
    type: str
    dflt: str


class Action(BaseModel):
    name: str
    roles: list[str]
    params: list[ActionParam]


class ActionsStore(BaseModel):
    store: str
    actions: list[Action]


class FormFieldInput(BaseModel):
    name: str
    value: str


class TaggedFeature(BaseModel):
    id: str
    tags: Tags
    lat: float
    lon: float


class TaggedLayer(BaseModel):
    store: str
    taggedFeatures: list[TaggedFeature]


class ActionResult(BaseModel):
    message: str | None = None
    taggedLayers: list[TaggedLayer] = []


class ActionResults(BaseModel):
    name: str | None = None
    message: str | None = None
    actionResults: list[ActionResult] = []


class ActionsResults(BaseModel):
    message: str | None = None
    actionResults: list[ActionResults] = []