from typing import Any, ClassVar
from sqlalchemy import BigInteger
from sqlalchemy.ext.mutable import MutableDict
from sqlalchemy.dialects.postgresql import HSTORE
from sqlmodel import Field, SQLModel, MetaData, JSON, TEXT, Relationship, Column
from pydantic import BaseModel, computed_field

from gisaf.models.metadata import gisaf
from gisaf.models.geo_models_base import GeoPointModel

class Tags(GeoPointModel, table=True):
    __table_args__ = gisaf.table_args
    hidden: ClassVar[bool] = True

    class Admin:
        menu = 'Other'
        flask_admin_model_view = 'TagModelView'

    id: int | None = Field(primary_key=True, default=None)
    store: str = Field(index=True)
    ref_id: int = Field(index=True, sa_type=BigInteger)
    tags: dict = Field(sa_type=MutableDict.as_mutable(HSTORE)) # type: ignore

    def __str__(self):
        return '{self.store:s} {self.ref_id}: {self.tags}'.format(self=self)

    def __repr__(self):
        return '<models.Tag {self.store:s} {self.ref_id}: {self.tags}>'.format(self=self)


class TagKey(SQLModel, table=True):
    __table_args__ = gisaf.table_args
    ## CREATE TABLE gisaf.tagkey (key VARCHAR(255) primary key);

    class Admin:
        menu = 'Other'
        flask_admin_model_view = 'TagKeyModelView'

    id: str | None = Field(default=None, primary_key=True)

    def __str__(self):
        return self.key

    def __repr__(self):
        return '<models.TagKey {self.key}>'.format(self=self)